@@ -87,6 +87,8 @@ class Scout(object):
         url = base_url + recon_type
         try:
             body = urllib2.urlopen(url, timeout=self.timeout).read()
+            if six.PY3 and isinstance(body, six.binary_type):
+                body = body.decode('utf8')
             content = json.loads(body)
             if self.verbose:
                 print("-> %s: %s" % (url, content))
@@ -129,7 +131,7 @@ class Scout(object):
             req = urllib2.Request(url)
             req.get_method = lambda: 'OPTIONS'
             conn = urllib2.urlopen(req)
-            header = conn.info().getheader('Server')
+            header = conn.info().get('Server')
             server_header = header.split('/')
             content = server_header[0]
             status = 200
@@ -167,8 +169,7 @@ class SwiftRecon(object):
             ret_dict = {'low': min(cstats), 'high': max(cstats),
                         'total': sum(cstats), 'reported': len(cstats),
                         'number_none': len(stats) - len(cstats), 'name': name}
-            ret_dict['average'] = \
-                ret_dict['total'] / float(len(cstats))
+            ret_dict['average'] = ret_dict['total'] / float(len(cstats))
             ret_dict['perc_none'] = \
                 ret_dict['number_none'] * 100.0 / len(stats)
         else:
@@ -220,7 +221,7 @@ class SwiftRecon(object):
         Compare ring md5sum's with those on remote host
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         :param swift_dir: The local directory with the ring files.
         """
         matches = 0
@@ -274,7 +275,7 @@ class SwiftRecon(object):
         Compare swift.conf md5sum with that on remote hosts
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         :param printfn: function to print text; defaults to print()
         """
         matches = 0
@@ -306,7 +307,7 @@ class SwiftRecon(object):
         Obtain and print async pending statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         scan = {}
         recon = Scout("async", self.verbose, self.suppress_errors,
@@ -328,7 +329,7 @@ class SwiftRecon(object):
         Obtain and print drive audit error statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)]
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)]
         """
         scan = {}
         recon = Scout("driveaudit", self.verbose, self.suppress_errors,
@@ -350,7 +351,7 @@ class SwiftRecon(object):
         Check for and print unmounted drives
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         unmounted = {}
         errors = {}
@@ -383,7 +384,7 @@ class SwiftRecon(object):
         Check for server types on the ring
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         errors = {}
         recon = Scout("server_type_check", self.verbose, self.suppress_errors,
@@ -406,7 +407,7 @@ class SwiftRecon(object):
         Obtain and print expirer statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         stats = {'object_expiration_pass': [], 'expired_last_pass': []}
         recon = Scout("expirer/%s" % self.server_type, self.verbose,
@@ -435,7 +436,7 @@ class SwiftRecon(object):
         Obtain and print replication statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         stats = {'replication_time': [], 'failure': [], 'success': [],
                  'attempted': []}
@@ -458,6 +459,8 @@ class SwiftRecon(object):
                         stats[stat_key].append(repl_stats.get(stat_key))
                 last = response.get('replication_last',
                                     response.get('object_replication_last', 0))
+                if last is None:
+                    continue
                 if last < least_recent_time:
                     least_recent_time = last
                     least_recent_url = url
@@ -501,7 +504,7 @@ class SwiftRecon(object):
         Obtain and print updater statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         stats = []
         recon = Scout("updater/%s" % self.server_type, self.verbose,
@@ -528,7 +531,7 @@ class SwiftRecon(object):
         Obtain and print obj auditor statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         scan = {}
         adone = '%s_auditor_pass_completed' % self.server_type
@@ -600,7 +603,7 @@ class SwiftRecon(object):
         Obtain and print obj auditor statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         all_scan = {}
         zbf_scan = {}
@@ -671,12 +674,75 @@ class SwiftRecon(object):
             print("[ZBF_auditor] - No hosts returned valid data.")
         print("=" * 79)
 
+    def sharding_check(self, hosts):
+        """
+        Obtain and print sharding statistics
+
+        :param hosts: set of hosts to check. in the format of:
+            set([('127.0.0.1', 6221), ('127.0.0.2', 6231)])
+        """
+        stats = {'sharding_time': [],
+                 'attempted': [], 'failure': [], 'success': []}
+        recon = Scout("sharding", self.verbose,
+                      self.suppress_errors, self.timeout)
+        print("[%s] Checking on sharders" % self._ptime())
+        least_recent_time = 9999999999
+        least_recent_url = None
+        most_recent_time = 0
+        most_recent_url = None
+        for url, response, status, ts_start, ts_end in self.pool.imap(
+                recon.scout, hosts):
+            if status == 200:
+                stats['sharding_time'].append(response.get('sharding_time', 0))
+                shard_stats = response.get('sharding_stats')
+                if shard_stats:
+                    # Sharding has a ton more stats, like "no_change".
+                    # Not sure if we need them at all, or maybe for -v.
+                    for stat_key in ['attempted', 'failure', 'success']:
+                        stats[stat_key].append(shard_stats.get(stat_key))
+                last = response.get('sharding_last', 0)
+                if last is None:
+                    continue
+                if last < least_recent_time:
+                    least_recent_time = last
+                    least_recent_url = url
+                if last > most_recent_time:
+                    most_recent_time = last
+                    most_recent_url = url
+        for k in stats:
+            if stats[k]:
+                computed = self._gen_stats(stats[k], name=k)
+                if computed['reported'] > 0:
+                    self._print_stats(computed)
+                else:
+                    print("[%s] - No hosts returned valid data." % k)
+            else:
+                print("[%s] - No hosts returned valid data." % k)
+        if least_recent_url is not None:
+            host = urlparse(least_recent_url).netloc
+            if not least_recent_time:
+                print('Oldest completion was NEVER by %s.' % host)
+            else:
+                elapsed = time.time() - least_recent_time
+                elapsed, elapsed_unit = seconds2timeunit(elapsed)
+                print('Oldest completion was %s (%d %s ago) by %s.' % (
+                    self._ptime(least_recent_time),
+                    elapsed, elapsed_unit, host))
+        if most_recent_url is not None:
+            host = urlparse(most_recent_url).netloc
+            elapsed = time.time() - most_recent_time
+            elapsed, elapsed_unit = seconds2timeunit(elapsed)
+            print('Most recent completion was %s (%d %s ago) by %s.' % (
+                self._ptime(most_recent_time),
+                elapsed, elapsed_unit, host))
+        print("=" * 79)
+
     def load_check(self, hosts):
         """
         Obtain and print load average statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         load1 = {}
         load5 = {}
@@ -705,7 +771,7 @@ class SwiftRecon(object):
         Obtain and print quarantine statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         objq = {}
         conq = {}
@@ -739,7 +805,7 @@ class SwiftRecon(object):
         Obtain and print /proc/net/sockstat statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         inuse4 = {}
         mem = {}
@@ -773,7 +839,7 @@ class SwiftRecon(object):
         Obtain and print disk usage statistics
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         stats = {}
         highs = []
@@ -872,7 +938,7 @@ class SwiftRecon(object):
         Check a time synchronization of hosts with current time
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         :param jitter: Maximal allowed time jitter
         """
 
@@ -911,7 +977,7 @@ class SwiftRecon(object):
         Check OS Swift version of hosts. Inform if differs.
 
         :param hosts: set of hosts to check. in the format of:
-            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+            set([('127.0.0.1', 6220), ('127.0.0.2', 6230)])
         """
         versions = set()
         errors = 0
@@ -984,7 +1050,8 @@ class SwiftRecon(object):
                         help="Print verbose info")
         args.add_option('--suppress', action="store_true",
                         help="Suppress most connection related errors")
-        args.add_option('--async', '-a', action="store_true",
+        args.add_option('--async', '-a',
+                        action="store_true", dest="async_check",
                         help="Get async stats")
         args.add_option('--replication', '-r', action="store_true",
                         help="Get replication stats")
@@ -994,6 +1061,8 @@ class SwiftRecon(object):
                         help="Get updater stats")
         args.add_option('--expirer', action="store_true",
                         help="Get expirer stats")
+        args.add_option('--sharding', action="store_true",
+                        help="Get sharding stats")
         args.add_option('--unmounted', '-u', action="store_true",
                         help="Check cluster for unmounted devices")
         args.add_option('--diskusage', '-d', action="store_true",
@@ -1086,6 +1155,7 @@ class SwiftRecon(object):
                 elif self.server_type == 'container':
                     self.auditor_check(hosts)
                     self.updater_check(hosts)
+                    self.sharding_check(hosts)
                 elif self.server_type == 'account':
                     self.auditor_check(hosts)
                 self.replication_check(hosts)
@@ -1102,7 +1172,7 @@ class SwiftRecon(object):
                 self.time_check(hosts, options.jitter)
                 self.version_check(hosts)
             else:
-                if options.async:
+                if options.async_check:
                     if self.server_type == 'object':
                         self.async_check(hosts)
                     else:
@@ -1129,7 +1199,14 @@ class SwiftRecon(object):
                     if self.server_type == 'object':
                         self.expirer_check(hosts)
                     else:
-                        print("Error: Can't check expired on non object "
+                        print("Error: Can't check expirer on non object "
+                              "servers.")
+                        print("=" * 79)
+                if options.sharding:
+                    if self.server_type == 'container':
+                        self.sharding_check(hosts)
+                    else:
+                        print("Error: Can't check sharding on non container "
                               "servers.")
                         print("=" * 79)
                 if options.validate_servers:
