@@ -42,6 +42,10 @@ warnings.filterwarnings('ignore', module='cryptography|OpenSSL', message=(
     'Python 2 is no longer supported by the Python core team. '
     'Support for it is now deprecated in cryptography, '
     'and will be removed in a future release.'))
+warnings.filterwarnings('ignore', message=(
+    'Python 3.6 is no longer supported by the Python core team. '
+    'Therefore, support for it is deprecated in cryptography '
+    'and will be removed in a future release.'))
 
 from eventlet.green import socket
 
