@@ -35,7 +35,7 @@ from swift.common.middleware.s3api.s3response import InvalidArgument, \
     AccessDenied, SignatureDoesNotMatch, RequestTimeTooSkewed, BadDigest
 from swift.common.utils import md5
 
-from test.unit import DebugLogger
+from test.debug_logger import debug_logger
 
 Fake_ACL_MAP = {
     # HEAD Bucket
@@ -110,7 +110,7 @@ class TestRequest(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         s3_req = req_klass(req.environ, conf=self.s3api.conf)
         s3_req.set_acl_handler(
-            get_acl_handler(s3_req.controller_name)(s3_req, DebugLogger()))
+            get_acl_handler(s3_req.controller_name)(s3_req, debug_logger()))
         with patch('swift.common.middleware.s3api.s3request.S3Request.'
                    '_get_response') as mock_get_resp, \
                 patch('swift.common.middleware.s3api.subresource.ACL.'
@@ -801,6 +801,11 @@ class TestRequest(S3ApiTestCase):
         self.assertEqual(expected_sts, sigv2_req._string_to_sign())
         self.assertTrue(sigv2_req.check_signature(secret))
 
+        with patch('swift.common.middleware.s3api.s3request.streq_const_time',
+                   return_value=True) as mock_eq:
+            self.assertTrue(sigv2_req.check_signature(secret))
+        mock_eq.assert_called_once()
+
     def test_check_signature_sigv2(self):
         self._test_check_signature_sigv2(
             'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY')
@@ -841,6 +846,12 @@ class TestRequest(S3ApiTestCase):
         self.assertFalse(sigv4_req.check_signature(
             u'\u30c9\u30e9\u30b4\u30f3'))
 
+        with patch('swift.common.middleware.s3api.s3request.streq_const_time',
+                   return_value=False) as mock_eq:
+            self.assertFalse(sigv4_req.check_signature(
+                u'\u30c9\u30e9\u30b4\u30f3'))
+        mock_eq.assert_called_once()
+
     @patch.object(S3Request, '_validate_dates', lambda *a: None)
     def test_check_signature_sigv4_unsigned_payload(self):
         environ = {
