@@ -22,12 +22,12 @@ import tarfile
 import zlib
 import mock
 import six
-from six import BytesIO
+from io import BytesIO
 from shutil import rmtree
 from tempfile import mkdtemp
 from eventlet import sleep
 from mock import patch, call
-from test.unit import debug_logger
+from test.debug_logger import debug_logger
 from test.unit.common.middleware.helpers import FakeSwift
 from swift.common import utils, constraints
 from swift.common.header_key_dict import HeaderKeyDict
